﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.ViewModel;
using AutomobilePlantDataModels.Enums;
using AutomobilePlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AutomobilePlantListImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public int CarId { get; private set; }
        public int ClientId { get; private set; }
        public string ClientName { get; private set; } = string.Empty;
        public int? ImplementerId { get; private set; }
        public string? ImplementerName { get; private set; } = string.Empty;
        public string CarName { get; private set; }
        public int Count { get; private set; }
        public double Sum { get; private set; }
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; private set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                CarId = model.CarId,
                ClientId = model.ClientId,
                CarName = model.CarName,
                ImplementerId = model.ImplementerId,
                ImplementerName = model.ImplementerName,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            CarId = CarId,
            ClientId=ClientId,
            ClientName = ClientName,
            ImplementerId=ImplementerId,
            ImplementerName=ImplementerName,
            CarName = CarName,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
    }
}
