﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.BusinessLogicsContracts;
using AutomobilePlantContracts.SearchModel;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace AutomobilePlant
{
    public partial class FormCreateOrder : Form
    {
        private readonly ILogger _logger;
        private readonly ICarLogic _logicCar;
        private readonly IOrderLogic _logicOrder;
        private readonly IClientLogic _logicClient;

        public FormCreateOrder(ILogger<FormCreateOrder> logger, ICarLogic logicCar, IOrderLogic logicOrder, IClientLogic logicClient)
        {
            InitializeComponent();
            _logger = logger;
            _logicCar = logicCar;
            _logicOrder = logicOrder;
            _logicClient = logicClient;
            LoadData();
        }

        private void LoadData()
        {
            _logger.LogInformation("Загрузка изделий для заказа");

            try
            {
                var listC = _logicClient.ReadList(null);
                var list = _logicCar.ReadList(null);
                if (list != null && listC!= null)
                {
                    CarComboBox.DisplayMember = "CarName";
                    CarComboBox.ValueMember = "Id";
                    CarComboBox.DataSource = list;
                    CarComboBox.SelectedItem = null;
                    
                    ClientcomboBox.DisplayMember = "Email";
                    ClientcomboBox.ValueMember = "Id";
                    ClientcomboBox.DataSource = listC;
                    ClientcomboBox.SelectedItem = null;
                }

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка изделий");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void FormCreateOrder_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void CalcSum()
        {
            if (CarComboBox.SelectedValue != null && !string.IsNullOrEmpty(CountTextBox.Text))
            {
                try
                {
                    int id = Convert.ToInt32(CarComboBox.SelectedValue);

                    var car = _logicCar.ReadElement(new CarSearchModel
                    {
                        Id = id
                    });

                    int count = Convert.ToInt32(CountTextBox.Text);
                    SumTextBox.Text = Math.Round(count * (car?.Price ?? 0), 2).ToString();
                    _logger.LogInformation("Расчет суммы заказа");
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка расчета суммы заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void CountTextBox_TextChanged(object sender, EventArgs e)
        {
            CalcSum();
        }

        private void CarComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            CalcSum();
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(CountTextBox.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (CarComboBox.SelectedValue == null)
            {
                MessageBox.Show("Выберите изделие", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Создание заказа");

            try
            {
                var operationResult = _logicOrder.CreateOrder(new OrderBindingModel
                {
                    CarId = Convert.ToInt32(CarComboBox.SelectedValue),
                    CarName = CarComboBox.Text,
                    ClientId = Convert.ToInt32(ClientcomboBox.SelectedValue),
                    ClientName = ClientcomboBox.Text,
                    Count = Convert.ToInt32(CountTextBox.Text),
                    Sum = Convert.ToDouble(SumTextBox.Text),
                }) ;

                if (!operationResult)
                {
                    throw new Exception("Ошибка при создании заказа. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания заказа");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
