﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.SearchModel;
using AutomobilePlantContracts.StoragesContracts;
using AutomobilePlantContracts.ViewModel;
using AutomobilePlantDataBaseImplements.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AutomobilePlantDataBaseImplements.Implements
{
    public class CarStorage : ICarStorage
    {
        public List<CarViewModel> GetFullList()
        {
            using var context = new AutoPlantDataBase();
            return context.Cars
                    .Include(x => x.Components)
                    .ThenInclude(x => x.Component)
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<CarViewModel> GetFilteredList(CarSearchModel model)
        {
            if (string.IsNullOrEmpty(model.CarName))
            {
                return new();
            }
            using var context = new AutoPlantDataBase();
            return context.Cars
                    .Include(x => x.Components)
                    .ThenInclude(x => x.Component)
                    .Where(x => x.CarName.Contains(model.CarName))
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public CarViewModel? GetElement(CarSearchModel model)
        {
            if (string.IsNullOrEmpty(model.CarName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new AutoPlantDataBase();
            return context.Cars
                .Include(x => x.Components)
                .ThenInclude(x => x.Component)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.CarName) && x.CarName == model.CarName) ||
                                (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }

        public CarViewModel? Insert(CarBindingModel model)
        {
            using var context = new AutoPlantDataBase();
            var newCar = Car.Create(context, model);
            if (newCar == null)
            {
                return null;
            }
            context.Cars.Add(newCar);
            context.SaveChanges();
            return newCar.GetViewModel;
        }

        public CarViewModel? Update(CarBindingModel model)
        {
            using var context = new AutoPlantDataBase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var car = context.Cars.FirstOrDefault(rec => rec.Id == model.Id);
                if (car == null)
                {
                    return null;
                }
                car.Update(model);
                context.SaveChanges();
                car.UpdateComponents(context, model);
                transaction.Commit();
                return car.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public CarViewModel? Delete(CarBindingModel model)
        {
            using var context = new AutoPlantDataBase();
            var element = context.Cars
                .Include(x => x.Components)
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Cars.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
