﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.SearchModel;
using AutomobilePlantContracts.StoragesContracts;
using AutomobilePlantContracts.ViewModel;
using AutomobilePlantDataBaseImplements.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AutomobilePlantDataBaseImplements.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new AutoPlantDataBase();
            return context.Orders
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (model is null)
            {
                return new();
            }
            using var context = new AutoPlantDataBase();
            if (model.DateTo != null && model.DateFrom != null && model.ClientId.HasValue)
            {
                return context.Orders
                    .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo && x.ClientId == model.ClientId)
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }

            else if (model.DateTo != null && model.DateFrom != null)
            {
                return context.Orders
                    .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.ClientId.HasValue)
            {
                return context.Orders
                    .Where(x => x.ClientId == model.ClientId)
                    .ToList()
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
                return context.Orders
                   .Where(x => x.Status == model.Status)
                   .ToList()
                   .Select(x => x.GetViewModel)
                   .ToList();
            
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new AutoPlantDataBase();
            if (model.ImplementerId.HasValue)
            {
                return context.Orders
                .FirstOrDefault(x =>  x.ImplementerId == model.ImplementerId && x.Status.Equals(model.Status))
                ?.GetViewModel;
            }
            return context.Orders
                .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)
                ?.GetViewModel;
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
           
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new AutoPlantDataBase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return newOrder.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new AutoPlantDataBase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var order = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
                if (order == null)
                {
                    return null;
                }
                order.Update(model);
                context.SaveChanges();
                transaction.Commit();
                return order.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new AutoPlantDataBase();
            var element = context.Orders
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

       
    }
}
