﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.ViewModel;
using AutomobilePlantDataModels.Enums;
using AutomobilePlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AutomobilePlantDataBaseImplements.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        [Required]
        public int CarId { get; private set; }
        [Required]
        public int ClientId { get; private set; }
        public string ClientName { get; private set; } = string.Empty;
        public int? ImplementerId { get; private set; }
        public string? ImplementerName { get; private set; } = string.Empty;
        public string CarName { get; private set; } = string.Empty;
        public Client client { get; private set; }
        public Car car { get; private set; }
        public Implementer? implementer { get; private set; }
        [Required]
        public int Count { get; private set; }
        [Required]
        public double Sum { get; private set; }
        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.SpecifyKind(DateTime.Now, DateTimeKind.Utc);
        public DateTime? DateImplement { get; private set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                CarId = model.CarId,
                ClientId = model.ClientId,
                CarName = model.CarName,
                ClientName = model.ClientName,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            ImplementerId = model.ImplementerId;
            ImplementerName = model.ImplementerName;
            Status = model.Status;
            DateImplement = model.DateImplement;
           
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            CarId = CarId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            ImplementerName=ImplementerName,
            CarName = CarName,
            ClientName = ClientName,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
    }
}
