﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.ViewModel;
using AutomobilePlantDataModels.Enums;
using AutomobilePlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace AutomomilePlantFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int CarId { get; private set; }
        public int ClientId { get; private set; }
        public string ClientName { get; private set; } = string.Empty;
        public string CarName  { get; private set; } = string.Empty;
        public int? ImplementerId { get; private set; }
        public string? ImplementerName { get; private set; } = string.Empty;
        public int Count { get; private set; }

        public double Sum { get; private set; }

        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

        public DateTime DateCreate { get; private set; } = DateTime.SpecifyKind(DateTime.Now, DateTimeKind.Utc);

        public DateTime? DateImplement { get; private set; } = null;

    public int Id { get; private set; }
        public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                CarId = model.CarId,
                CarName = model.CarName,
                Count = model.Count,
                ImplementerName = model.ImplementerName,
                ImplementerId = model.ImplementerId,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                Id = model.Id
            };
        }
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                CarId = Convert.ToInt32(element.Element("CarId")!.Value),
                CarName= element.Element("CarName")!.Value,
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                ClientName = element.Element("ClientName")!.Value,
                ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
                ImplementerName = element.Element("ImplementerName")!.Value,
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum= Convert.ToDouble(element.Element("Sum")!.Value),
                Status= (OrderStatus)Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value),
                DateCreate=DateTime.Parse(element.Element("DateCreate")!.Value),
                DateImplement= string.IsNullOrEmpty(element.Element("DateImplement")!.Value) ? null : DateTime.Parse(element.Element("DateImplement")!.Value)

            };
        }
        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;

        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            CarId = CarId,
            CarName = CarName,
            ClientId = ClientId,
            ClientName = ClientName,
            ImplementerId = ImplementerId,
            ImplementerName = ImplementerName,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
        public XElement GetXElement => new("Order",
        new XAttribute("Id", Id),
        new XElement("CarId", CarId.ToString()),
        new XElement("CarName", CarName),
        new XElement("Count", Count.ToString()),
        new XElement("Sum", Sum.ToString()),
        new XElement("Status", Status.ToString()),
        new XElement("DateCreate", DateCreate.ToString()),
        new XElement("DateImplement", DateImplement.ToString())
        );
    }
}
